<?php

/**
 * @file
 * Views listing and editing single view.
 *
 * This file contains a lot of functions resembling the views_ui.module
 * & includes/admin.inc from the views UI module. I did this to
 * a) change the path of the submissions etc.
 * b) have better control over complete access control.
 * c) control display of templates when necessary.
 */

/**
 * Page callback to list views or show edit page.
 */
function views_ui_perm_views_page($view = NULL) {
  if (!empty($view->disabled)) {
    drupal_set_message(t('You have no permission to edit a disabled view.'), 'error');
    drupal_goto('views_ui_perm');
  }
  if (!empty($view)) {
    drupal_set_title(t('Edit view "%view"', array('%view' => $view->name)));
    $output = theme('views_ui_perm_edit_view', $view);
    return $output;
  }
  else {
    $output = theme('views_ui_perm_list_views');
    return $output;
  }
}

/**
 * Generic entry point to handle forms.
 */
function views_ui_perm_ajax_form($js, $key, &$view, $display_id) {
  $path_views = drupal_get_path('module', 'views');
  require_once("./$path_views/includes/admin.inc");

  $form = views_ui_ajax_forms($key);
  if (empty($form)) {
    return drupal_not_found();
  }

  views_include('ajax');
  $args = func_get_args();
  // Remove the known args
  array_splice($args, 0, 4);

  $form_state = views_ui_build_form_state($js, $key, $view, $display_id, $args);
  // check to see if this is the top form of the stack. If it is, pop
  // it off; if it isn't, the user clicked somewhere else and the stack is
  // now irrelevant.
  if (!empty($view->stack)) {
    $identifier = views_ui_build_identifier($key, $view, $display_id, $args);
    $top = array_shift($view->stack);
    if (array_shift($top) != $identifier) {
      $view->stack = array();
    }
  }

  $output = views_ajax_form_wrapper($form_state['form_id'], $form_state);

  if (!$output) {
    // Sometimes we need to re-generate the form for multi-step type operations.
    $object = NULL;
    if (!empty($view->stack)) {
      $stack = $view->stack; // copy so the next shift doesn't break the array
      $top = array_shift($stack);
      $top[0] = $js; // change identifier into $js setting
      $form_state = call_user_func_array('views_ui_build_form_state', $top);
      $form_state['input'] = array(); // this is a new form, make sure it
      // doesn't try to inherit $_POST info.
      if (!$js) {
        return drupal_goto(views_ui_perm_build_form_url($form_state));
      }
      $object = views_ajax_form_wrapper($form_state['form_id'], $form_state);
      $object->url = url(views_ui_perm_build_form_url($form_state));
    }
    else if (!$js) {
      // if nothing on the stack, non-js forms just go back to the main view editor.
      return drupal_goto("views_ui_perm/$view->name");
    }
    // regenerate all tabs because changes to the default tab could ripple.
    return views_ui_regenerate_tabs($view, NULL, $object);
  }

  return ($js) ? views_ajax_render($output) : $output;
}

/**
 * Create the URL for one of our standard AJAX forms based upon known
 * information about the form.
 */
function views_ui_perm_build_form_url($form_state) {
  $form = views_ui_ajax_forms($form_state['form_key']);
  $ajax = empty($form_state['ajax']) ? 'nojs' : 'ajax';
  $name = $form_state['view']->name;
  $url = "views_ui_perm/$ajax/$form_state[form_key]/$name/$form_state[display_id]";
  foreach ($form['args'] as $arg) {
    $url .= '/'. $form_state[$arg];
  }
  return $url;
}

/**
 * Submit handler for submitting the edit view form.
 */
function views_ui_perm_edit_view_form_submit($form, &$form_state) {
  $form_state['view']->save();
  drupal_set_message(t('The view has been saved.'));

  // Make sure menu items get rebuilt as neces
  menu_rebuild();

  // Clear the views cache.
  cache_clear_all('*', 'cache_views');

  // Clear the page cache.
  cache_clear_all();

  // Remove this view from cache so we can edit it properly.
  views_object_cache_clear('view', $form_state['view']->name);

  // Use drupal_goto to redirect to our edit view.
  drupal_goto('views_ui_perm/'. $form_state['view']->name);
}

/**
 * Submit handler for cancelling the edit view form.
 */
function views_ui_perm_edit_view_form_cancel($form, &$form_state) {
  // Remove this view from cache so edits will be lost.
  views_object_cache_clear('view', $form_state['view']->name);
  if (empty($form['view']->vid)) {
    drupal_goto('views_ui_perm');
  }
}

/**
 * Generate the summary output for a single display to render in a tab.
 */
function views_ui_perm_display_tab($view, $display) {
  if (isset($display->handler)) {
    $plugin = $display->handler->definition;
  }
  if (empty($plugin)) {
    $title = isset($display->display_title) ? $display->display_title : t('Invalid');
    return array($title, t("Error: Display @display refers to a plugin named '@plugin', but that plugin doesn't exist!", array('@display' => $display->id, '@plugin' => $display->display_plugin)));

    // @todo We can do a better 'plugin does not exist' tab.
  }

  // The display should always be initialized prior to this call.
  if (empty($display->handler)) {
    return FALSE;
  }

  $body = theme('views_ui_perm_edit_tab', $view, $display);
  return array($display->display_title, $body);
}

/**
 * Template preprocess, show list of views.
 * Do not show disabled or default views.
 */
function template_preprocess_views_ui_perm_list_views(&$vars) {
  $path_views = drupal_get_path('module', 'views');
  require_once("./$path_views/includes/admin.inc");

  $views = views_get_all_views();

  $base_tables = views_fetch_base_tables();

  foreach ($views as $view) {
    if (empty($view->disabled)) {

      $item = new stdClass();
      $item->ops = array();
      if (empty($view->disabled)) {
        $item->ops[] = l(t('Edit'), "views_ui_perm/$view->name");
      }

      $item->ops = implode(' | ', $item->ops);
      if (empty($view->display)) {
        $item->path = t('Warning! Broken view!');
      }
      else {
        $item->path = $raw_path = $view->get_path();
        $item->path = $item->path && empty($view->disabled) && strpos($item->path, '%') === FALSE ? l($item->path, $item->path) : check_plain($item->path);
      }

      $item->type = $view->type;
      $item->name = $view->name;

      if (!empty($view->tag)) {
        $item->tag = $view->tag;
      }

      $item->title = $view->get_title();
      $item->base = !empty($base_tables[$view->base_table]['title']) ? $base_tables[$view->base_table]['title'] : t('Broken');

      $item->displays = array();
      foreach ($view->display as $display) {
        if (!empty($display->handler->definition['admin'])) {
          $item->displays[$display->handler->definition['admin']] = TRUE;
        }
      }

      if ($item->displays) {
        ksort($item->displays);
        $item->displays = implode(', ', array_keys($item->displays));
      }

      $item->description = check_plain($view->description);

      $items[] = $item;

      switch ($form_state['values']['order']) {
        case 'name':
        default:
          $sorts[] = drupal_strtolower($view->name);
          break;
        case 'title':
          $sorts[] = drupal_strtolower($title);
          break;
        case 'path':
          $sorts[] = drupal_strtolower($raw_path); // $path;
          break;
        case 'type':
          $sorts[] = $view->type . $view->name;
          break;
        case 'tag':
          $sorts[] = drupal_strtolower($view->tag);
          break;
        case 'desc':
          $sorts[] = drupal_strtolower($view->description);
          break;
      }
    }
  }

  asort($sorts);

  $i = array();
  foreach ($sorts as $id => $title) {
    $i[] = $items[$id];
  }

  views_add_css('views-list');
  $vars['views'] = $i;
}

/**
 * Preprocess the view edit page.
 */
function template_preprocess_views_ui_perm_edit_view(&$vars) {
  $path_views = drupal_get_path('module', 'views');
  require_once("./$path_views/includes/admin.inc");
  require_once("./$path_views/includes/base.inc");
  require_once("./$path_views/includes/plugins.inc");

  $view = &$vars['view'];

  $vars['save_button'] = drupal_get_form('views_ui_edit_view_form', $view);

  $table = views_fetch_data($view->base_table);
  $vars['base_table'] = !empty($table['table']['base']['title']) ?
    $table['table']['base']['title'] : t('Unknown or missing table name');

  views_include('tabs');
  $tabs = new views_tabset;

  $vars['message'] = '<div class="message">'. t("Click on an item to edit that item's details.") .'</div>';

  if (!$view->set_display('default')) {
    drupal_set_message(t('This view has a broken default display and cannot be used.'), 'error');
  }

  foreach ($view->display as $display) {
    list($title, $body) = views_ui_perm_display_tab($view, $display);
    // The first display is the default.
    $tabs->set($display->id, $title, $body);
  }

  // This is the area that will render beneath the links
  $form_state = array(
    'view' => &$view,
    'ajax' => FALSE,
  );

  $display_button = drupal_build_form('views_ui_add_display_form', $form_state);
  $analyze_button = drupal_get_form('views_ui_analyze_view_button', $view);
  $tabs->add_extra($display_button . $analyze_button);

  $vars['tabs'] = $tabs->render();

  $form_state = array(
    'display_id' => 'default',
    'view_args' => '',
    'rerender' => FALSE,
    'no_redirect' => TRUE,
    'view' => &$view,
    'input' => array(),
  );
  $vars['preview'] = drupal_build_form('views_ui_preview_form', $form_state);

  $vars['locked'] = NULL;
  if (isset($view->locked) && is_object($view->locked)) {
    $account = user_load($view->locked->uid);
    $vars['locked'] = theme('username', $account);
    $vars['lock_age'] = format_interval(time() - $view->locked->updated);
    $vars['break'] = l(t('break this lock'), 'admin/build/views/break-lock/'. $view->name);
  }

  $paths = array();
  foreach ($view->display as $id => $display) {
    if (!empty($display->handler) && $display->handler->has_path()) {
      $path = $display->handler->get_path();
      if (strpos($path, '%') === FALSE && !isset($paths[$path])) {
        $vars['quick_links_raw'][] = array(
          'title' => t('View "!display"', array('!display' => $display->display_title)),
          'alt' => t("Go to the real page for this display"),
          'href' => $path,
        );
        // Displays can have the same path; no point in showing more than one link.
        $paths[$path] = TRUE;
      }
    }
  }

  $vars['quick_links'] = theme('links', $vars['quick_links_raw']);
  views_add_css('views-admin');
  views_add_css('views');
  views_add_js('ajax');
  drupal_add_js('misc/jquery.form.js');

  // Also add any js files required by plugins:
  $plugins = views_fetch_plugin_data();
  foreach ($plugins as $type => $type_plugins) {
    foreach ($type_plugins as $name => $plugin) {
      if (!empty($plugin['js'])) {
        foreach ($plugin['js'] as $file) {
          drupal_add_js($file);
        }
      }
    }
  }

  $settings = array('views' => array('ajax' => array(
    'id' => '#views-ajax-pad',
    'title' => '#views-ajax-title',
    'defaultForm' => $vars['message'],
  )));

  drupal_add_js($settings, 'setting');
}

/**
 * Preprocess the view edit tab.
 */
function template_preprocess_views_ui_perm_edit_tab(&$vars) {
  $path_views = drupal_get_path('module', 'views');
  require_once("./$path_views/includes/admin.inc");

  $view = $vars['view'];
  $display = $vars['display'];
  $plugin = $display->handler->definition;

  $top = $left = $middle = $right = '';

  // If this form was submitted it was already handled, so force it not to
  // submit again.

  $vars['remove'] = '';
  if (empty($plugin['no remove'])) {
    if (!empty($_POST['form_id']) && $_POST['form_id'] == 'views_ui_remove_display_form') {
      unset($_POST['form_id']);
    }
    $form_state = array('view' => &$view, 'display_id' => $display->id, 'ajax' => FALSE);
    $vars['remove'] = drupal_build_form('views_ui_remove_display_form', $form_state);
  }

  // basic fields
  $vars['title'] = check_plain($display->display_title);
  $vars['description'] = check_plain($plugin['help']);

  // Special fields if tihs is the default display.
  $vars['default'] = ($display->id == 'default');
  $vars['details_class'] = views_ui_item_css('details');
  if (!empty($view->changed_sections['details'])) {
    $vars['details_changed'] = TRUE;
  }

  $tag = empty($view->tag) ? t('None') : $view->tag;
  $vars['details'] = t('Tag') .':'. l($tag, "views_ui_perm/nojs/details/$view->name", array('attributes' => array('class' => 'views-ajax-link')));

  // Calculate options from display plugin.
  $options = $categories = array();
  $display->handler->options_summary($categories, $options);

  // Build all of the options we were returned and put them into the
  // category data fields.
  foreach ($options as $id => $option) {
    if (empty($categories[$option['category']]['data'])) {
      $categories[$option['category']]['data'] = array();
    }
    $categories[$option['category']]['data'][$id] = array();
    $data = &$categories[$option['category']]['data'][$id];
    $data['content'] = '';
    $data['links'] = '';
    $data['overridden'] = FALSE;
    $data['defaulted'] = FALSE;

    // If there are optional links, build them first so they float properly.
    if (!empty($option['links'])) {
      foreach ($option['links'] as $link_id => $link_value) {
        $data['links'] .= $display->handler->option_link($link_value, $link_id, 'views-button-configure');
      }
    }
    if (!empty($option['title'])) {
      $data['content'] .= $option['title'] .': ';
    }

    $data['content'] .= $display->handler->option_link($option['value'], $id, '', empty($option['desc']) ? '' : $option['desc']);
    if (!empty($display->handler->options['defaults'][$id])) {
      $display_id = 'default';
      $data['defaulted'] = TRUE;
    }
    else {
      $display_id = $display->id;
      if (!$display->handler->is_default_display()) {
        if ($display->handler->defaultable_sections($id)) {
          $data['overridden'] = TRUE;
        }
      }
    }
    $data['class'] = views_ui_item_css($display_id .'-'. $id);
    if (!empty($view->changed_sections[$display_id .'-'. $id])) {
      $data['changed'] = TRUE;
    }
  }

  $vars['categories'] = $categories;

  // Add a help icon
  if (isset($plugin['help topic'])) {
    $vars['display_help_icon'] = theme('advanced_help_topic', $plugin['module'], $plugin['help topic']);
  }
  else {
    $vars['display_help_icon'] = '';
  }

  // Fetch style plugin info because it has some effect on how/what we render.
  $style_plugin = $display->handler->get_plugin();

  // Check which items we want to display.
  $options = views_ui_perm_available_fields();
  $ui_permissions = variable_get('views_ui_permissions', views_ui_perm_default_values($options));
  if ($ui_permissions['field'] != '0')
  $vars['fields'] = theme('views_ui_perm_edit_item', 'field', $view, $display, !($style_plugin && $style_plugin->uses_fields()));
  else $vars['fields'] = '';
  if ($ui_permissions['relationship'] != '0')
  $vars['relationships'] = theme('views_ui_perm_edit_item', 'relationship', $view, $display);
  else $vars['relationships'] = '';
  if ($ui_permissions['argument'] != '0')
  $vars['arguments'] = theme('views_ui_perm_edit_item', 'argument', $view, $display);
  else $vars['arguments'] = '';
  if ($ui_permissions['filter'] != '0')
  $vars['filters'] = theme('views_ui_perm_edit_item', 'filter', $view, $display);
  else $vars['filters'] = '';
  if ($ui_permissions['sort'] != '0')
  $vars['sorts'] = theme('views_ui_perm_edit_item', 'sort', $view, $display);
  else $vars['sorts'] = '';
}

/**
 * Add information about a section to a display.
 */
function template_preprocess_views_ui_perm_edit_item(&$vars) {
  $type = $vars['type'];
  $view = $vars['view'];
  $display = $vars['display'];

  $types = views_object_types();

  $vars['overridden'] = FALSE;
  $vars['defaulted'] = FALSE;

  if ($vars['no_fields']) {
    $vars['title'] = $types[$type]['title'];
    $vars['item_help_icon'] = theme('advanced_help_topic', 'views', $type);
    $vars['rearrange'] = NULL;
    $vars['add'] = NULL;
    return;
  }

  $vars['rearrange'] = l('<span>'. t('Rearrange') .'</span>', "views_ui_perm/nojs/rearrange/$view->name/$display->id/$type", array('attributes' => array('class' => 'views-button-rearrange views-ajax-link', 'title' => t('Rearrange')), 'html' => TRUE));

  $vars['add'] = l('<span>'. t('Add') .'</span>', "views_ui_perm/nojs/add-item/$view->name/$display->id/$type", array('attributes' => array('class' => 'views-button-add views-ajax-link', 'title' => t('Add')), 'html' => TRUE));

  if (!$display->handler->is_default_display()) {
    if (!$display->handler->is_defaulted($types[$type]['plural'])) {
      $vars['overridden'] = TRUE;
    }
    else {
      $vars['defaulted'] = TRUE;
    }
  }

  $vars['title'] = l($types[$type]['title'], "views_ui_perm/nojs/config-type/$view->name/$display->id/$type", array('attributes' => array('class' => 'views-ajax-link')));
//  $vars['title'] = l($types[$type]['title'], "admin/build/views/nojs/config-type/$view->name/$display->id/$type", array('attributes' => array('class' => 'views-ajax-link')));

  $fields = array();

  static $relationships = NULL;
  if (!isset($relationships)) {
    // Get relationship labels
    $relationships = array();
    // @todo: get_handlers()
    foreach ($display->handler->get_option('relationships') as $id => $relationship) {
      $handler = views_get_handler($relationship['table'], $relationship['field'], 'relationship');
      if (empty($handler)) {
        continue;
      }
      $handler->init($view, $relationship);
      $relationships[$id] = $handler->label();
    }
  }

  // @todo: get_handlers()
  foreach ($display->handler->get_option($types[$type]['plural']) as $id => $field) {
    $fields[$id] = array();

    $handler = views_get_handler($field['table'], $field['field'], $type);
    if (empty($handler)) {
      $fields[$id]['class'] = 'broken';
      $fields[$id]['title'] = t("Error: handler for @table > @field doesn't exist!", array('@table' => $field['table'], '@field' => $field['field']));
      $fields[$id]['info'] = '';
      continue;
    }
    $handler->init($view, $field);

    $field_name = $handler->ui_name();
    if (!empty($field['relationship']) && !empty($relationships[$field['relationship']])) {
      $field_name = '('. $relationships[$field['relationship']] .') '. $field_name;
    }

    $fields[$id]['title'] = l($field_name, "views_ui_perm/nojs/config-item/$view->name/$display->id/$type/$id", array('attributes' => array('class' => 'views-ajax-link')));
    $fields[$id]['class'] = views_ui_item_css($display->id .'-'. $type .'-'. $id);
    if (!empty($view->changed_sections[$display->id .'-'. $type .'-'. $id])) {
      $fields[$id]['changed'] = TRUE;
    }
    $fields[$id]['info'] = $handler->admin_summary();

    if ($handler->has_extra_options()) {
      $fields[$id]['links'] = l('<span>'. t('Settings') .'</span>', "views_ui_perm/nojs/config-item-extra/$view->name/$display->id/$type/$id", array('attributes' => array('class' => 'views-button-configure views-ajax-link', 'title' => t('Settings')), 'html' => TRUE));
    }

    if ($handler->needs_style_plugin()) {
      $style_plugin = views_fetch_plugin_data('style', $handler->options['style_plugin']);
      $style_title = empty($style_plugin['title']) ? t('Missing style plugin') : $style_plugin['title'];
      $pid = $id .'-style-plugin';

      if (!empty($style_plugin['uses options'])) {
        $fields[$pid]['links'] = l('<span>'. t('Change settings for this style') .'</span>', "views_ui_perm/nojs/config-style/$view->name/$display->id/$type/$id", array('attributes' => array('class' => 'views-button-configure views-ajax-link', 'title' => t('Settings')), 'html' => TRUE));
      }

      $fields[$pid]['title'] = t(' &nbsp; Style: !style', array('!style' => l($style_title, "views_ui_perm/nojs/change-style/$view->name/$display->id/$type/$id", array('attributes' => array('class' => 'views-ajax-link')))));
      $fields[$pid]['class'] = views_ui_item_css($display->id .'-'. $type .'-'. $pid);
      if (!empty($view->changed_sections[$display->id .'-'. $type .'-'. $pid])) {
        $fields[$pid]['changed'] = TRUE;
      }
      $fields[$pid]['info'] = '';
    }
  }

  $vars['fields'] = $fields;
  $vars['item_help_icon'] = theme('advanced_help_topic', 'views', $type);
}
